@@ -11,9 +11,11 @@ function(panel.data,	## REQUIRED
 	chunk.size=5000,
 	convert.0and100=TRUE,
 	projection.unit="YEAR",
-	percentile.trajectory.values=1:99,
+	percentile.trajectory.values=NULL,
 	isotonize=TRUE,
-	projcuts.digits=2) {
+	projcuts.digits=0) {
+
+	started.at=proc.time()
 
 	##########################################################
 	###
@@ -158,9 +160,9 @@ function(panel.data,	## REQUIRED
 			tmp.traj <- reshape(data.table(tmp.traj, CUT=rep(percentile.trajectory.values, dim(tmp.traj)[1]/length(percentile.trajectory.values))), 
 				idvar="ID", timevar="CUT", direction="wide")
 			if (projection.unit=="GRADE") {
-				names(tmp.traj) <- c("ID", do.call(paste, c(expand.grid("P", percentile.trajectory.values, "_PROJ_SS", grade.projection.sequence), sep="")))
+				names(tmp.traj) <- c("ID", do.call(paste, c(expand.grid("P", percentile.trajectory.values, "_PROJ_GRADE_", grade.projection.sequence), sep="")))
 			} else {
-				names(tmp.traj) <- c("ID", do.call(paste, c(expand.grid("P", percentile.trajectory.values, "_PROJ_YEAR", seq_along(grade.projection.sequence)), sep="")))
+				names(tmp.traj) <- c("ID", do.call(paste, c(expand.grid("P", percentile.trajectory.values, "_PROJ_YEAR_", seq_along(grade.projection.sequence)), sep="")))
 			}
 			key(tmp.traj) <- "ID"
 			if (!cuts.tf) return(tmp.traj)
@@ -177,9 +179,9 @@ function(panel.data,	## REQUIRED
 					tmp <- tmp.cutscores[[paste("GRADE_", grade.projection.sequence[i], sep="")]][j]
 					cuts.arg[k] <- paste(".sgp.targets(SS", grade.projection.sequence[i], ", ", tmp, ", ", convert.0and100, ")", sep="")
 					if (projection.unit=="GRADE") {
-						names.arg[k] <- paste("LEVEL_", j, "_TARGET_SS", grade.projection.sequence[i], sep="")
+						names.arg[k] <- paste("LEVEL_", j, "SGP_TARGET_GRADE_", grade.projection.sequence[i], sep="")
 					} else {
-						names.arg[k] <- paste("LEVEL_", j, "_TARGET_YEAR", i, sep="")
+						names.arg[k] <- paste("LEVEL_", j, "SGP_TARGET_YEAR_", i, sep="")
 					}
 					k <- k+1
 				}
@@ -224,7 +226,7 @@ function(panel.data,	## REQUIRED
 			stop("Please specify an appropriate list of SGP function labels (sgp.labels). See help page for details.")
 		}
 		if (!identical(names(sgp.labels), c("my.year", "my.subject")) & 
-			!identical(names(sgp.labels), c("my.year", "my.subject", "my.grade"))) {
+			!identical(names(sgp.labels), c("my.year", "my.subject", "my.extra.label"))) {
 			stop("Please specify an appropriate list for sgp.labels. See help page for details.")
 			}
 			tmp.path <- .create.path(sgp.labels)
@@ -235,26 +237,38 @@ function(panel.data,	## REQUIRED
 	}
 
 	if (!missing(use.my.knots.boundaries)) {
-		if (!is.list(use.my.knots.boundaries)) {
-			stop("Please specify an appropriate list for use.my.knots.boundaries. See help page for details.")
+		if (!is.list(use.my.knots.boundaries) & !is.character(use.my.knots.boundaries)) {
+			stop("use.my.knots.boundaries must be supplied as a list or character abbreviation. See help page for details.")
 		}
-		if (!identical(names(use.my.knots.boundaries), c("my.year", "my.subject")) &
-			!identical(names(use.my.knots.boundaries), c("my.year", "my.subject", "my.grade"))) {
-			stop("Please specify an appropriate list for use.my.knots.boundaries. See help page for details.")
+		if (is.list(use.my.knots.boundaries)) {
+			if (!(class(panel.data) %in% c("list", "SGP"))) {
+				stop("use.my.knots.boundaries is only appropriate when panel data is of class list or SGP. See help page for details.")
+			}
+			if (!identical(names(use.my.knots.boundaries), c("my.year", "my.subject")) & 
+				!identical(names(use.my.knots.boundaries), c("my.year", "my.subject", "my.extra.label"))) {
+					stop("Please specify an appropriate list for use.my.knots.boundaries. See help page for details.")
 			}
 			tmp.path.knots.boundaries <- .create.path(use.my.knots.boundaries)
-			if (is.null(panel.data$Knots_Boundaries) | is.null(panel.data$Knots_Boundaries[[tmp.path.knots.boundaries]])) {
-				stop("Knots and Boundaries indicated by argument use.my.knots.boundaries are not included.")
-		}} else {
-			tmp.path.knots.boundaries <- tmp.path
+			if (is.null(panel.data[["Knots_Boundaries"]]) | is.null(panel.data[["Knots_Boundaries"]][[tmp.path.knots.boundaries]])) {
+				stop("Knots and Boundaries indicated by use.my.knots.boundaries are not included.")
+			}
 		}
+			if (is.character(use.my.knots.boundaries)) {
+				if (!use.my.knots.boundaries %in% names(stateData)) {
+					stop("Knots and Boundaries are currently not implemented for the state indicated. Please contact the SGP package administrator to have your Knots and Boundaries included in the package")
+		}
+     		tmp.path.knots.boundaries <- tmp.path    
+		}
+	} else {
+     		tmp.path.knots.boundaries <- tmp.path
+	}
 
 	if (!missing(use.my.coefficient.matrices)) {
 		if (!is.list(use.my.coefficient.matrices)) {
 			stop("Please specify an appropriate list for use.my.coefficient.matrices. See help page for details.")
 		}
-		if (!identical(names(use.my.coefficient.matrices), c("my.year", "my.subject")) |
-			!identical(names(use.my.coefficient.matrices), c("my.year", "my.subject", "my.grade"))) {
+		if (!identical(names(use.my.coefficient.matrices), c("my.year", "my.subject")) &
+			!identical(names(use.my.coefficient.matrices), c("my.year", "my.subject", "my.extra.label"))) {
 			stop("Please specify an appropriate list for use.my.coefficient.matrices. See help page for details.")
 			}
 			tmp.path.coefficient.matrices <- .create.path(use.my.coefficient.matrices)
@@ -373,14 +387,15 @@ function(panel.data,	## REQUIRED
 
 	### Announce Completion & Return SGP Object
 
-	print(paste("Finished Growth Projection Analysis ", date(), ". Subject: ", sgp.labels$my.subject, ", Year: ", sgp.labels$my.year, ", Grade Progression: ", paste(grade.progression, collapse=", "), sep="")) 
+	message(paste("Finished SGP Student Growth Projection/Trajectory Analysis ", date(), " in ", timetaken(started.at), ".\nSubject: ", sgp.labels$my.subject, ", Year: ", sgp.labels$my.year, ", Grade Progression: ", paste(grade.progression, collapse=", "), " ", sgp.labels$my.extra.label, sep="")) 
 
-	list(Coefficient_Matrices=panel.data$Coefficient_Matrices,
+	list(Coefficient_Matrices=panel.data[["Coefficient_Matrices"]],
 		Cutscores=Cutscores,
-		Goodness_of_Fit=panel.data$Goodness_of_Fit, 
-		Knots_Boundaries=panel.data$Knots_Boundaries, 
-		Panel_Data=panel.data$Panel_Data,
-		SGPercentiles=panel.data$SGPercentiles,
-		SGProjections=SGProjections)
+		Goodness_of_Fit=panel.data[["Goodness_of_Fit"]], 
+		Knots_Boundaries=panel.data[["Knots_Boundaries"]], 
+		Panel_Data=panel.data[["Panel_Data"]],
+		SGPercentiles=panel.data[["SGPercentiles"]],
+		SGProjections=SGProjections,
+		Simulated_SGPs=panel.data[["Simulated_SGPs"]])
 
 } ## END studentGrowthProjections Function
